// Flutter imports:
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

// Package imports:
import 'package:flutter_screenutil/flutter_screenutil.dart';

// Project imports:
import 'package:zego_uikit/src/components/defines.dart';
import 'package:zego_uikit/src/components/internal/internal.dart';
import 'package:zego_uikit/src/service/service.dart';

class ZegoInRoomMessageInput extends StatefulWidget {
  const ZegoInRoomMessageInput({
    Key? key,
    this.placeHolder = "Say something...",
    this.backgroundColor,
    this.borderRadius,
    this.enabled = true,
    this.onSubmit,
  }) : super(key: key);

  final String placeHolder;
  final Color? backgroundColor;
  final double? borderRadius;
  final bool enabled;
  final VoidCallback? onSubmit;

  @override
  State<ZegoInRoomMessageInput> createState() => _ZegoInRoomMessageInputState();
}

class _ZegoInRoomMessageInputState extends State<ZegoInRoomMessageInput> {
  final TextEditingController textController = TextEditingController();
  ValueNotifier<bool> isEmptyNotifier = ValueNotifier(true);

  @override
  Widget build(BuildContext context) {
    var messageSendBgColor = const Color(0xff000000).withOpacity(0.25);

    return ConstrainedBox(
      constraints: BoxConstraints(
        minHeight: 90.r,
      ),
      child: Container(
        decoration: BoxDecoration(
          color: widget.backgroundColor ?? messageSendBgColor,
          borderRadius: BorderRadius.vertical(
            top: Radius.circular(widget.borderRadius ?? 24.r),
          ),
        ),
        alignment: Alignment.center,
        child: Row(
          children: [
            messageInput(),
            sendButton(),
            SizedBox(width: 20.r),
          ],
        ),
      ),
    );
  }

  Widget messageInput() {
    var messageSendCursorColor = const Color(0xffA653ff);
    var messageSendHintStyle = TextStyle(
      color: const Color(0xFFFFFFFF).withOpacity(0.2),
      fontSize: 28.r,
      fontWeight: FontWeight.w400,
    );
    var messageSendInputStyle = TextStyle(
      color: Colors.white,
      fontSize: 28.r,
      fontWeight: FontWeight.w400,
    );

    return Expanded(
      child: TextField(
        enabled: widget.enabled,
        keyboardType: TextInputType.multiline,
        minLines: 1,
        maxLines: null,
        autofocus: true,
        inputFormatters: <TextInputFormatter>[
          LengthLimitingTextInputFormatter(400)
        ],
        controller: textController,
        onChanged: (String inputMessage) {
          bool valueIsEmpty = inputMessage.isEmpty;
          if (valueIsEmpty != isEmptyNotifier.value) {
            isEmptyNotifier.value = valueIsEmpty;
          }
        },
        textInputAction: TextInputAction.send,
        onSubmitted: (message) => send(),
        cursorColor: messageSendCursorColor,
        cursorHeight: 30.r,
        cursorWidth: 3.r,
        style: messageSendInputStyle,
        onTap: () {},
        decoration: InputDecoration(
          hintText: widget.placeHolder,
          hintStyle: messageSendHintStyle,
          isDense: true,
          contentPadding: EdgeInsets.fromLTRB(30.r, 25.r, 20.r, 25.r),
          border: InputBorder.none,
        ),
      ),
    );
  }

  Widget sendButton() {
    return ValueListenableBuilder<bool>(
      valueListenable: isEmptyNotifier,
      builder: (context, bool isEmpty, Widget? child) {
        return ZegoTextIconButton(
          onPressed: () {
            if (!isEmpty) send();
          },
          icon: ButtonIcon(
            icon: isEmpty
                ? UIKitImage.asset(StyleIconUrls.iconSendDisable)
                : UIKitImage.asset(StyleIconUrls.iconSend),
            backgroundColor: const Color(0xff1e2740).withOpacity(0.4),
          ),
          iconSize: Size(40.r, 40.r),
          buttonSize: Size(72.r, 72.r),
        );
      },
    );
  }

  void send() {
    if (textController.text.isEmpty) {
      debugPrint("message is empty");
      return;
    }

    ZegoUIKit().sendInRoomMessage(textController.text);
    textController.clear();

    widget.onSubmit?.call();
  }
}
